#!/usr/local/bin/ruby -Ks

# }CXC[pm ver.2.0
#  gui.rb: GUIŎs
# Copyright 2007 again@T
#  http://nijzero.dw.land.to/

require 'Win32API'
require 'jcode'
require 'yaml'

require 'vr/vruby'
require 'vr/vrcontrol'
require "vr/vrtimer"

require 'screen'
require 'textreader'
require 'calcprob'

$debug = false if $debug == nil
#$debug = true

module MPForm
  MIN = 0
  LOW = 10
  MIDDLE = 20
  HIGH = 50
  MAX = 100
  COLOR = {MIN => [255,0,0], LOW => [255,255,0], MIDDLE => [0,255,0], HIGH => [0,255,255], MAX => [0,0,255]}
  
  include VRTimerFeasible
  include VRDrawable
  
  def self_paint
    setPen(RGB(0,0,0))
    setBrush(RGB(0,0,0),SWin::BRUSH::NULL)
    fillRoundRect(5,10,200,160,10,10)
    fillRoundRect(205,10,380,160,10,10)
  end
  
  def construct
    self.caption = "}CXC[pm"
    defaultfont = @screen.factory.newfont("lr oSVbN",14)
    @yamlfiles = Dir.glob("*.yml").collect do |yamlfilename|
      {"filename" => yamlfilename, "name" => YAML.load_file(yamlfilename)["name"]}
    end
    if @yamlfiles.empty?
      messageBox("ʒ`t@C(YAMLt@C)܂B","}CXC[pm:G[")
      exit
    end
    
    addControl(VRStatic,"lbl3"," }CXC[pʂ",20,5,150,15)
    @lbl3.setFont(defaultfont)
    addControl(VRButton,"btn_byscreen","mvZ",15,25,175,25)
    @btn_byscreen.setFont(defaultfont)
    addControl(VRStatic,"lbl2","IvV",15,60,80,15)
    @lbl2.setFont(defaultfont)
    addControl(VRCheckbox,"chklogfile","ڍׂt@Cɏo",15,80,170,15)
    @chklogfile.setFont(defaultfont)
    addControl(VRCombobox,"combo_windowmode","",15,100,175,100)
    @combo_windowmode.setFont(defaultfont)
    @combo_windowmode.setListStrings(@yamlfiles.collect{|file| file["name"]})
    @combo_windowmode.select(0)
    addControl(VRButton,"btn_remind","măvbg",15,130,175,25)
    @btn_remind.setFont(defaultfont)
    
    addControl(VRStatic,"lbl4"," eLXgt@C",220,5,130,15)
    @lbl4.setFont(defaultfont)
    addControl(VRButton,"btn_file","t@CImvZ",215,25,155,25)
    @btn_file.setFont(defaultfont)
    
    addControl(VRStatic,"lbl1","m",9,168,40,15)
    @lbl1.setFont(defaultfont)
    addControl(VREdit,"edt_prob_result","",40,165,50,20)
    @edt_prob_result.setFont(defaultfont)
    @edt_prob_result.readonly = true
    addControl(VRText,"logtext","",5,185,380,80)
    @logtext.setFont(@screen.factory.newfont("lr SVbN",12))
    @logtext.readonly = true
    
    addTimer(300,"t1")
    
    class << @logtext
      def init(debug)
        @debug = debug
        @stored_text = ""
      end
      def write(str)
        self.text += str + "\n"
        self.setCaret(self.text.length)
        self.scrolltocaret
        self.scrolldownPage
      end
      def dump(str)
        if @debug
          self.text += str + "\n"
          self.setCaret(self.text.length)
          self.scrolltocaret
          self.scrolldownPage
        end
      end
      def store(str)
        @stored_text.concat(str + "\n")
      end
      def write_file(writeok)
        if writeok
          filename = Time.now.strftime("%Y-%m-%d_%H-%M-%S.txt")
          f = File.open(filename, "w")
          begin
            f.write(@stored_text)
          rescue
          ensure
            f.close
          end
        end
        @stored_text = ""
      end
    end
    
    @logtext.init($debug)
  end
  
  def t1_timer
    begin
      x,y = @si.current_cell_pos
      @edt_prob_result.text = pcent = @pc.percents[y][x].round.to_s + "%"
      sleep(0.002)
    rescue
      nil
    end
  end
  
  def btn_byscreen_clicked
    begin
      @logtext.text = ""
      
      @si = ScreenIO.new(@yamlfiles[@combo_windowmode.selectedString]["filename"])
      @si.cell_values
      @si.rest_value
      data = @si
    rescue => evar
      messageBox(evar.to_s,"}CXC[pm:G[")
      return nil
    end
    
    begin
      deleteTimer("t1")
      @pc = ProbabilityCalculator.new(data.cell, data.rest, @logtext)
      @si.set_colors(@pc.percents)
    rescue => evar
      @logtext.store(evar.to_s)
      messageBox("m߂܂łB","}CXC[pm:G[")
    ensure
      @logtext.write_file(@chklogfile.checked?)
      addTimer(300,"t1")
    end
  end
  
  def btn_remind_clicked
    begin
      @si.set_colors(@pc.percents)
    rescue => evar
      messageBox(evar.to_s,"}CXC[pm:G[")
    end
  end
  
  def btn_file_clicked
    if (filename=openFilenameDialog([ ["eLXgt@C(*.txt)","*.txt"],["ׂẴt@C(*.*)","*.*"] ]))
      begin
        data = TextParser.new(filename)
        data.cell.each do |line|
          @logtext.dump(line.join(" "))
        end
        @logtext.dump("rows:#{data.rows}")
        @logtext.dump("cols:#{data.cols}")
        @logtext.dump("rest:#{data.rest}")
        pc = ProbabilityCalculator.new(data.cell, data.rest, @logtext)
        pc.percents.each do |line|
          linestr = line.collect{|val| val.to_s_2figures}
          @logtext.dump(stre = linestr.join(" "))
          @logtext.store(stre)
        end
      rescue => evar
        @logtext.write(evar.to_s)
        @logtext.store(evar.to_s)
      ensure
        @logtext.write_file(true)
      end
    end
  end
  
  def prob_to_color(prob)
    bgr = ""
    if prob < LOW
      higher, lower = LOW, MIN
    elsif prob < MIDDLE
      higher, lower = MIDDLE, LOW
    elsif prob < HIGH
      higher, lower = HIGH, MIDDLE
    else
      higher, lower = MAX, HIGH
    end
    0.upto(2) do |index|
      hexval = (COLOR[lower][index]*(higher-prob)/(higher-lower) + COLOR[higher][index]*(prob-lower)/(higher-lower)).round.to_s(16)
      bgr.concat((hexval.length<2) ? ("0"+hexval) : hexval)
    end
    return bgr.hex
  end
end

class ControlWindow
  WM_SETICON = 128
  ICON_SMALL = 0
  
  def initialize
    #icon= VRLocalScreen.factory.iconfromfile("mineprob.ico")
    #MyForm = VRLocalScreen.newFormClass("LocalScreen",nil,nil,icon,nil)
    #class MyForm
    #  def construct
    #    self.caption="test"
    #  end
    #end
    frm=VRLocalScreen.showForm(MPForm,0,0,400,300)
    extractIconA = Win32API.new('shell32','ExtractIconA','LPI','L')
    iconfile = defined?(ExerbRuntime) ? ExerbRuntime.filename : "mineprob.ico"
    iconData = extractIconA.Call(0, iconfile, 0)
    frm.sendMessage(WM_SETICON, ICON_SMALL, iconData) if iconData != 0
    
    VRLocalScreen.messageloop
  end
end

ControlWindow.new

